@@ -1,5 +1,5 @@
 """
-Copyright (C) 2013-2017 Stefan Pfenninger.
+Copyright (C) 2013-2017 Calliope contributors listed in AUTHORS.
 Licensed under the Apache 2.0 License (see LICENSE file).
 
 analysis.py
@@ -20,8 +20,6 @@ except ImportError:
 import numpy as np
 import pandas as pd
 
-from . import core
-from . import utils
 from . import analysis_utils as au
 
 
@@ -46,11 +44,14 @@ def plot_carrier_production(solution, carrier='power', subset=dict(),
     return plot_timeseries(solution, data, carrier=carrier, demand='demand_{}'.format(carrier), **kwargs)
 
 
-def plot_timeseries(solution, data, carrier='power', demand='demand_power',
-                    types=['supply', 'conversion', 'storage', 'unmet_demand'],
-                    colormap=None, ticks=None,
-                    resample_options=None, resample_func=None,
-                    add_legend=True, ax=None):
+def plot_timeseries(
+        solution, data, carrier='power', demand='demand_power',
+        tech_types=[
+            'supply', 'supply_plus', 'conversion',
+            'conversion_plus', 'storage', 'unmet_demand'],
+        colormap=None, ticks=None,
+        resample_options=None, resample_func=None,
+        add_legend=True, ax=None):
     """
     Generate a stackplot of ``data`` for the given ``carrier``,
     plotting ``demand`` on top.
@@ -68,9 +69,9 @@ def plot_timeseries(solution, data, carrier='power', demand='demand_power',
     demand : str, optional
         Name of a demand tech whose time series to plot on top,
         default 'demand_power'.
-    types : list, optional
+    tech_types : list, optional
         Technology types to include in the plot. Default list is
-        ['supply', 'conversion', 'storage', 'unmet_demand'].
+        ['supply', 'supply_plus' , 'conversion', 'storage', 'unmet_demand'].
     colormap : matplotlib colormap, optional
         Colormap to use. If not given, the colors specified for each
         technology in the solution's metadata are used.
@@ -105,10 +106,18 @@ def plot_timeseries(solution, data, carrier='power', demand='demand_power',
     plot_df = data.divide(time_res, axis='index')
     if resample_options and resample_func:
         plot_df = getattr(plot_df.resample(**resample_options), resample_func)()
+    df_metadata = solution['metadata'].to_pandas()
+    # Get pandas DataFrame based on carrier being plotted
+    df = df_metadata.query('carrier_in == "{}" or '
+                           'carrier_out == "{}"'.format(carrier, carrier))
+    # Specifically add pertinent conversion_plus technologies to df
+    if 'conversion_plus' in tech_types:
+        df_cp = df_metadata.query('type == "conversion_plus"')
+        for y, columns in df_cp.iterrows():
+            if carrier in columns.carrier_in or carrier in columns.carrier_out:
+                df = df.append(df_metadata.loc[y])
     # Get tech stack and names
-    df = solution['metadata'].to_pandas().query('carrier == "{}"'.format(carrier))
-    query_string = au._get_query_string(types)
-    stacked_techs = df.query(query_string).index.tolist()
+    stacked_techs = df_metadata.query('type in {}'.format(tech_types)).index.tolist()
     # Put stack in order according to stack_weights
     weighted = df.stack_weight.sort_values(ascending=False).index.tolist()
     stacked_techs = [y for y in weighted if y in stacked_techs]
@@ -134,27 +143,35 @@ def plot_timeseries(solution, data, carrier='power', demand='demand_power',
     return ax
 
 
-def plot_installed_capacities(solution,
-                              types=['supply', 'conversion', 'storage'],
-                              unit_multiplier=1.0,
-                              unit_label='kW',
-                              **kwargs):
+def plot_installed_capacities(
+        solution,
+        tech_types=[
+            'supply', 'supply_plus', 'conversion',
+            'conversion_plus', 'storage'
+        ],
+        unit_multiplier=1.0,
+        unit_label='kW',
+        **kwargs):
     """
     Plot installed capacities (``e_cap``) with a bar plot.
 
     Parameters
     ----------
-    types : list, default ['supply', 'conversion', 'storage']
-        Technology types to include in the plot.
-    unit_multiplier : float or int, default 1.0
+    solution : model solution xarray.Dataset
+    tech_types : list, optional
+        Technology types to include in the plot. Default is
+        ['supply', 'supply_plus', 'conversion', 'conversion_plus', 'storage']
+    unit_multiplier : float or int, optional
         Multiply installed capacities by this value for plotting.
-    unit_label : str, default 'kW'
-        Label for capacity values, adjust this when
-        changing ``unit_multiplier``.
-    **kwargs : are passed to ``pandas.DataFrame.plot()``
+        Defaults to 1.0
+    unit_label : str, optional
+        Label for capacity values. Default is 'kW', adjust this
+        when changing ``unit_multiplier``.
+    **kwargs : optional
+        are passed to ``pandas.DataFrame.plot()``
 
     """
-    query_string = au._get_query_string(types)
+    query_string = au._get_query_string(tech_types)
     md = solution.metadata.to_pandas()
     supply_cap = md.query(query_string).index.tolist()
 
@@ -192,7 +209,7 @@ def plot_installed_capacities(solution,
     return ax
 
 
-def plot_transmission(solution, tech='hvac', carrier='power',
+def plot_transmission(solution, tech='ac_transmission', carrier='power',
                       labels='utilization',
                       figsize=(15, 15), fontsize=9,
                       show_scale=True,
@@ -208,7 +225,7 @@ def plot_transmission(solution, tech='hvac', carrier='power',
     Parameters
     ----------
     solution : solution container
-    tech : str, default 'hvac'
+    tech : str, default 'ac_transmission'
         Which transmission technology to plot.
     carrier : str, default 'power'
         Which carrier to plot transmission for.
@@ -230,7 +247,8 @@ def plot_transmission(solution, tech='hvac', carrier='power',
     # Determine maximum that could have been transmitted across a link
     def get_edge_capacity(solution, a, b):
         hrs = solution['time_res'].to_pandas().sum()
-        cap = solution['e_cap_net'].loc[dict(x=a, y='{}:'.format(tech) + b)].to_pandas() * hrs
+        cap = (solution['e_cap_net'].loc[dict(x=a, y='{}:'.format(tech) + b)]
+                                    .to_pandas() * hrs)
         return cap
 
     # Get annual power transmission between zones
@@ -238,7 +256,8 @@ def plot_transmission(solution, tech='hvac', carrier='power',
     trans_tech = lambda x: '{}:{}'.format(tech, x)
     def get_annual_power_transmission(zone):
         try:
-            return solution['e'].loc[dict(c=carrier, y=trans_tech(zone))].sum(dim='t').to_pandas()
+            return (solution['e'].loc[dict(c=carrier, y=trans_tech(zone))]
+                                .sum(dim='t').to_pandas())
         except KeyError:
             return 0
     df = pd.DataFrame({zone: get_annual_power_transmission(zone)
@@ -302,13 +321,14 @@ def get_delivered_cost(solution, cost_class='monetary', carrier='power',
     """
     summary = solution.summary.to_pandas()
     meta = solution.metadata.to_pandas()
-    carrier_subset = meta[meta.carrier == carrier].index.tolist()
+    carrier_subset = meta[meta.carrier_out == carrier].index.tolist()
     if count_unmet_demand is False:
         try:
             carrier_subset.remove('unmet_demand_' + carrier)
         except ValueError:  # no unmet demand technology
             pass
-    cost = solution['costs'].loc[dict(k=cost_class, y=carrier_subset)].to_pandas().sum().sum()
+    cost = (solution['costs'].loc[dict(k=cost_class, y=carrier_subset)]
+                             .to_pandas().sum().sum())
     # Actually, met_demand also includes demand "met" by unmet_demand
     met_demand = summary.at['demand_' + carrier, 'e_con']
     try:
@@ -324,21 +344,22 @@ def get_delivered_cost(solution, cost_class='monetary', carrier='power',
 
 
 def get_levelized_cost(solution, cost_class='monetary', carrier='power',
-                       group=None, locations=None,
+                       groups=None, locations=None,
                        unit_multiplier=1.0):
     """
     Get the levelized cost per unit of energy produced for the given
     ``cost_class`` and ``carrier``, optionally for a subset of technologies
-    given by ``group`` and a subset of ``locations``.
+    given by ``groups`` and a subset of ``locations``.
 
     Parameters
     ----------
     solution : solution container
     cost_class : str, default 'monetary'
     carrier : str, default 'power'
-    group : str, default None
-        Limit the computation to members of the given group (see the
-        groups table in the solution for valid groups).
+    groups : list, default None
+        Limit the computation to members of the given groups (see the
+        groups table in the solution for valid groups). Defaults to
+        ['supply', 'supply_plus'] if not given.
     locations : str or iterable, default None
         Limit the computation to the given location or locations.
     unit_multiplier : float or int, default 1.0
@@ -347,10 +368,15 @@ def get_levelized_cost(solution, cost_class='monetary', carrier='power',
         ``unit_multiplier=0.001`` will return cost per MWh.
 
     """
-    if group is None:
-        group = 'supply'
+    if groups is None:
+        groups = ['supply', 'supply_plus']
 
-    members = solution.groups.to_pandas().at[group, 'members'].split('|')
+    members = []
+    for g in groups:
+        try:
+            members.extend(solution.groups.to_pandas().at[g, 'members'].split('|'))
+        except KeyError:
+            pass
 
     if locations is None:
         locations_slice = slice(None)
@@ -361,37 +387,36 @@ def get_levelized_cost(solution, cost_class='monetary', carrier='power',
         locations_slice = locations
 
     cost = solution['costs'].loc[dict(k=cost_class, x=locations_slice, y=members)]
-    ec_prod = solution['ec_prod'].loc[dict(c=carrier, x=locations_slice, y=members)]
+    c_prod = solution['c_prod'].loc[dict(c=carrier, x=locations_slice, y=members)]
 
     if locations is None:
         cost = cost.sum(dim='x').to_pandas()
-        ec_prod = ec_prod.sum(dim='x').to_pandas()
+        c_prod = c_prod.sum(dim='x').to_pandas()
     else:
         cost = cost.to_pandas()
-        ec_prod = ec_prod.to_pandas()
+        c_prod = c_prod.to_pandas()
 
-    return (cost / ec_prod) * unit_multiplier
+    return (cost / c_prod) * unit_multiplier
 
 
-def get_group_share(solution, techs, group_type='supply',
-                    var='e_prod'):
+def get_group_share(solution, techs, group, var='e_prod'):
     """
     From ``solution.summary``, get the share of the given list of ``techs``
-    from the total for the given ``group_type``, for the given ``var``.
+    from the total for the given ``group``, for the given ``var``.
 
     """
     summary = solution.summary
     meta = solution.metadata.to_pandas()
-    group = meta.query('type == "' + group_type + '"').index.tolist()
-    if group_type == 'transmission':
+    members = meta.query('type == "' + group + '"').index.tolist()
+    if group == 'transmission':
         # Special case for transmission techs, we only want
         # the base tech names, since we're looking this
         # up in the summary table
         def transmission_basenames(group):
             return list(set([i.split(':')[0] for i in group]))
         techs = transmission_basenames(techs)
-        group = transmission_basenames(group)
-    supply_total = summary.loc[group, var].sum()
+        members = transmission_basenames(members)
+    supply_total = summary.loc[members, var].sum()
     supply_group = summary.loc[techs, var].sum()
     try:
         with np.errstate(divide='ignore', invalid='ignore'):
@@ -496,7 +521,7 @@ def get_domestic_supply_index(solution):
     # TODO: add unit tests
     idx = solution.metadata.query('type == "supply"').index.tolist()
     dom = (solution.costs.loc[dict(k='domestic', y=idx)].sum().sum() /
-           solution['ec_prod'].loc[dict(c='power')].sum().sum())
+           solution['c_prod'].loc[dict(c='power')].sum().sum())
     return dom
 
 
@@ -525,141 +550,3 @@ def map_results(results, func, as_frame=False):
         return pd.DataFrame(items, index=idx)
     else:
         return pd.Series(items, index=idx)
-
-
-class SolutionModel(core.BaseModel):
-    def __init__(self, solution):
-        """
-        Dummy model created from a model solution,
-        which gives access to the ``config_model`` AttrDict and a subset
-        of Model methods, including ``get_option``, ``get_cost``, and
-        ``get_depreciation``.
-
-        Furthermore, it provides several ``recompute_`` methods
-        to recalculate costs for a given technology after modifying the
-        underlying data.
-
-        """
-        super().__init__()
-        self.config_model = solution.config_model
-
-        # Reconstruct self.data from solution
-        params = ['r', 'e_eff']
-        try:
-            self.data = solution[params]
-        except KeyError:
-            pass
-
-        # Attach solution (minus data parameters)
-        for p in params:
-            try:
-                del solution[p]
-            except KeyError:
-                pass
-        self.solution = solution
-
-        # Add getters
-        self._get_option = utils.option_getter(self.config_model)
-        self._get_cost = utils.cost_getter(self._get_option)
-        self._get_dep = utils.depreciation_getter(self._get_option)
-
-    def get_option(self, *args, **kwargs):
-        return self._get_option(*args, **kwargs)
-
-    def get_cost(self, *args, **kwargs):
-        return self._get_cost(*args, **kwargs)
-
-    def get_depreciation(self, *args, **kwargs):
-        return self._get_dep(*args, **kwargs)
-
-    def recompute_investment_costs(self, y, k='monetary',
-                                   cost_adjustment=1.0):
-        """
-        Recompute investment costs for the given technology ``y``.
-
-        ``cost_adjustment`` allows linear scaling of the costs by the
-        given amount.
-
-        """
-        get_cost = self.get_cost
-        solution = self.solution
-        cost_con = pd.DataFrame()
-        cost_con['e_cap'] = (get_cost('e_cap', y, k)
-                             * solution['e_cap'].loc[dict(y=y)])
-        cost_con['r_cap'] = (get_cost('r_cap', y, k)
-                             * solution['r_cap'].loc[dict(y=y)])
-        cost_con['r_area'] = (get_cost('r_area', y, k)
-                              * solution['r_area'].loc[dict(y=y)])
-        cost_con['rb_cap'] = (get_cost('rb_cap', y, k)
-                              * solution['rb_cap'].loc[dict(y=y)])
-        cost_con['s_cap'] = (get_cost('s_cap', y, k)
-                             * solution['s_cap'].loc[dict(y=y)])
-        cost_con['total'] = cost_con.sum(axis=1)
-        cost_con['total_per_e_cap'] = (cost_con['total']
-                                       / solution['e_cap'].loc[dict(y=y)])
-        cost_con.at['total', 'total'] = cost_con['total'].sum()
-        total_per_e_cap = (cost_con.at['total', 'total']
-                           / solution['e_cap'].loc[dict(y=y)].sum())
-        cost_con.at['total', 'total_per_e_cap'] = total_per_e_cap
-        return cost_con * cost_adjustment
-
-    def recompute_operational_costs(self, y, k='monetary', carrier='power',
-                                    cost_adjustment=1.0):
-        """
-        Recompute operational costs for the given technology ``y``.
-
-        ``cost_adjustment`` linearly scales the construction costs used
-        internally, but is not applied to the rest of the operational
-        costs.
-
-        """
-        get_cost = self.get_cost
-        get_depreciation = self.get_depreciation
-        solution = self.solution
-        # Here we want gross production
-        production = solution['es_prod'].loc[dict(c=carrier, y=y)]
-        fuel = (solution['rs'].loc[dict(y=y)].sum(dim='t')
-                / self.get_option(y + '.constraints.r_eff'))
-        fuel_rb = (solution['rbs'].loc[dict(y=y)].sum(dim='t')
-                   / self.get_option(y + '.constraints.rb_eff'))
-        icf = self.recompute_investment_costs
-        cost_con = icf(y, k=k, cost_adjustment=cost_adjustment)['total'].sum()
-        cost_op = pd.DataFrame()
-        cost_op['om_frac'] = (get_cost('om_frac', y, k) * cost_con
-                              * get_depreciation(y, k)
-                              * (solution['time_res'].to_pandas().sum() / 8760))
-        cost_op['om_fixed'] = (get_cost('om_fixed', y, k)
-                               * solution['e_cap'].loc[dict(y=y)]
-                               * (solution['time_res'].to_pandas().sum() / 8760))
-        cost_op['om_var'] = get_cost('om_var', y, k) * production
-        cost_op['om_fuel'] = get_cost('om_fuel', y, k) * fuel
-        cost_op['om_rb'] = get_cost('om_rb', y, k) * fuel_rb
-        cost_op['total'] = cost_op.sum(axis=1)
-        return cost_op
-
-    def recompute_levelized_costs(self, y, k='monetary', carrier='power',
-                                  cost_adjustment=1.0):
-        """
-        Recompute levelized costs for the given technology ``y``.
-
-        ``cost_adjustment`` linearly scales the construction costs used
-        internally to compute levelized costs.
-
-        """
-        get_depreciation = self.get_depreciation
-        solution = self.solution
-        icf = self.recompute_investment_costs
-        cost_con = icf(y, k=k, cost_adjustment=cost_adjustment)['total']
-        ocf = self.recompute_operational_costs
-        cost_op = ocf(y, k=k, carrier=carrier,
-                      cost_adjustment=cost_adjustment)['total']
-        # Here we want net production
-        production = solution['ec_prod'].loc[dict(c=carrier, y=y)]
-        lc = (get_depreciation(y, k) * (solution['time_res'].to_pandas().sum() / 8760)
-              * cost_con + cost_op) / production.to_pandas()
-        lc.at['total'] = (get_depreciation(y, k)
-                          * (solution['time_res'].to_pandas().sum() / 8760)
-                          * cost_con.total
-                          # * cost_con.drop(['total'], axis=0).sum()
-                          + cost_op.sum()) / production.sum()
-        return lc
