@@ -1,5 +1,5 @@
 """
-Copyright (C) 2013-2017 Stefan Pfenninger.
+Copyright (C) 2013-2017 Calliope contributors listed in AUTHORS.
 Licensed under the Apache 2.0 License (see LICENSE file).
 
 planning.py
@@ -11,7 +11,7 @@ Planning constraints.
 
 import numpy as np
 import pyomo.core as po  # pylint: disable=import-error
-
+from . import base
 
 def node_constraints_build_total(model):
     """
@@ -48,21 +48,23 @@ def system_margin(model):
     m = model.m
     time_res = model.data['_time_res'].to_series()
 
-    def carrier(y):
-        return model.get_option(y + '.carrier')
-
+    def carrier(y, x):
+        if y in m.y_conversion_plus:
+            return model.get_cp_carriers(y, x)[0]
+        else:
+            return model.get_option(y + '.carrier', default= y + '.carrier_out')
     # Constraint rules
     def c_system_margin_rule(m, c):
         # If no margin defined for a carrier, use 0 (i.e. no margin)
         margin = model.config_model.system_margin.get_key(c, default=0)
         if margin:
             t = model.t_max_demand[c]
-            return (sum(m.es_prod[c, y, x, t] for y in m.y for x in m.x)
-                    * (1 + margin)
-                    <= time_res.at[t]
-                    * sum((m.e_cap[y, x] / model.get_eff_ref('e', y, x))
-                          for y in m.y if carrier(y) == c
-                          for x in m.x))
+            return (sum(m.c_prod[c, y, x, t] for y in m.y if y not in m.y_demand
+                        for x in m.x) * (1 + margin)
+                    <= time_res.at[t] *
+                    sum((m.e_cap[y, x] / base.get_constraint_param(m, 'e_eff', y, x, t))
+                         for y in m.y if y not in m.y_demand
+                         for x in m.x if carrier(y, x) == c))
         else:
             return po.Constraint.NoConstraint
 
