@@ -1,5 +1,5 @@
 """
-Copyright (C) 2013-2017 Stefan Pfenninger.
+Copyright (C) 2013-2017 Calliope contributors listed in AUTHORS.
 Licensed under the Apache 2.0 License (see LICENSE file).
 
 utils.py
@@ -450,7 +450,7 @@ def option_getter(config_model):
                     return _get_option(default, fail)
                 # 'ccgt.constraints.s_time' -> 'ccgt', 'constraints.s_time'
                 tech, remainder = opt.split('.', 1)
-                if ':' in tech:
+                if ':' in tech: # transmission
                     parent = tech.split(':')[0]
                 else:
                     # parent = e.g. 'defaults'
@@ -464,7 +464,10 @@ def option_getter(config_model):
                                 'with given default '
                                 '`{}`'.format(option, default))
                     elif default:
-                        result = _get_option(default, fail=True)
+                        if not isinstance(default, str): #allow setting the default directly as anything that isn't a string - could do with being more robust
+                            result = default
+                        else:
+                            result = _get_option(default, fail=True)
                     elif tech == 'defaults':
                         raise e('Reached top of inheritance chain '
                                 'and no default defined for: '
@@ -498,25 +501,37 @@ def option_getter(config_model):
     return get_option
 
 
-def cost_getter(option_getter_func, costs_type='costs'):
-    def get_cost(cost, y, k, x=None):
-        return option_getter_func(
-            y + '.' + costs_type + '.' + k + '.' + cost,
-            default=y + '.' + costs_type + '.default.' + cost,
-            x=x)
+def cost_getter(option_getter_func):
+    def get_cost(cost, y, k, x=None, costs_type='costs'):
+        return option_getter_func(y + '.' + costs_type + '.' + k + '.' + cost,
+                             default=y + '.' + costs_type + '.default.' + cost,
+                             x=x)
     return get_cost
 
 
-def cost_per_distance_getter(option_getter_func):
+def cost_per_distance_getter(config_model):
+    option_getter_func = option_getter(config_model)
     def get_cost_per_distance(cost, y, k, x):
+        _cost = cost_getter(option_getter_func)
+        cost = _cost(cost, y, k, x, costs_type='costs_per_distance')
+        tech, x2 = y.split(':')
+        per_distance = option_getter_func(y + '.per_distance')
+        link = config_model.get_key('links.'+ x + ',' + x2,
+            default=config_model['links'].get(x2 + ',' + x))
+        # link = None if no link exists
+        if not link or tech not in link.keys():
+            return 0
         try:
-            cost_option = y + '.costs_per_distance.' + k + '.' + cost
-            cost = option_getter_func(cost_option)
-            per_distance = option_getter_func(y + '.per_distance')
-            distance = option_getter_func(y + '.distance', x=x)
-            distance_cost = cost * (distance / per_distance)
-        except exceptions.OptionNotSetError:
-            distance_cost = 0
+            distance = link.get_key(tech + '.distance')
+        except KeyError:
+            if cost > 0:
+                e = exceptions.OptionNotSetError
+                raise e('Distance must be defined for link: {} '
+                        'and transmission tech: {}, as cost_per_distance '
+                        'is defined'.format(x + ',' + x2, tech))
+            else:
+                return 0
+        distance_cost = cost * (distance / per_distance)
         return distance_cost
     return get_cost_per_distance
 
@@ -538,24 +553,51 @@ def depreciation_getter(option_getter_func):
 
 def any_option_getter(model):
     """
-    Get any option from the given Model or SolutionModel, including
+    Get any option from the given Model, including
     ``costs.`` or ``costs_per_distance.`` options
 
     """
     get_cost = cost_getter(model.get_option)
-    get_cost_pd = cost_getter(model.get_option, 'costs_per_distance')
+    get_cost_pd = cost_per_distance_getter(model.get_option)
 
-    def get_any_option(option):
+    def get_any_option(option, x=None, t=None):
         if 'costs.' in option:
-            y, rest, k, cost = option.split('.')
-            return get_cost(cost, y, k)
-        elif 'costs_per_distance.' in option:
-            y, rest, k, cost = option.split('.')
-            return get_cost_pd(cost, y, k)
+            if t and x:
+                y, cost_type, k, cost = option.split('.')
+                try:
+                    return model.data['_'.join([cost_type,k,cost])].sel(y=y,x=x,t=t)
+                except:
+                    return model.get_cost(cost, y, k, x=x, costs_type=cost_type)
+            elif t and not x:
+                e = exceptions.OptionNotSetError
+                raise e('must define location for time dependant variable '
+                    '`{}`'.format(option))
+            else:
+                y, cost_type, k, cost = option.split('.')
+                return model.get_cost(cost, y, k, x=x, costs_type=cost_type)
         else:
-            return model.get_option(option)
+            if t and x:
+                y = option.split('.', 1)[0]
+                field = option.rsplit('.', 1)[-1]
+                try:
+                    return model.data[field].sel(y=y,x=x,t=t)
+                except:
+                    return model.get_option(option, x=x)
+            elif t and not x:
+                e = exceptions.OptionNotSetError
+                raise e('must define location for time dependant variable '
+                        '`{}`'.format(option))
+            elif x and not t:
+                return model.get_option(option, x=x)
+            else:
+                if 'costs_per_distance.' in option:
+                    y, rest, k, cost = option.split('.')
+                    return get_cost_pd(cost, y, k)
+                else:
+                    return model.get_option(option)
     return get_any_option
 
+
 def vincenty(coord1, coord2):
     """
     Vincenty's inverse method formula to calculate the distance in metres
